@@ -1,6 +1,6 @@
 import asyncio
-from typing import Callable, List, TYPE_CHECKING
 from enum import Enum
+from typing import Callable, List, TYPE_CHECKING, Dict, Any
 
 if TYPE_CHECKING:
     from .api import IngeniumAPI
@@ -119,29 +119,40 @@ class IngComponentType(Enum):
     def is_busing_rgb_color_picker(self): return self in [self.COD_RGB]
 
 
-class IngComponent:
-    label: str
-    address: int
-    output: int
-    type: IngComponentType
-    icon: int
+# noinspection PyBroadException
+def safecast(value, cast_fn):
+    try:
+        return cast_fn(value)
+    except BaseException:
+        return None
 
-    real1: float
+
+def get_device_type(ctype) -> IngComponentType:
+    try:
+        return IngComponentType(safecast(ctype, int))
+    except ValueError:
+        print("Invalid component: " + str(ctype))
+        return IngComponentType.NOT_IMPLEMENTED
+
+
+class IngComponent:
+    id: str
+    label: str
+    output: int
+    icon: int
 
     _notify_update: List[Callable] = []
 
+    def __init__(self, json: Dict[str, Any] = None):
+        if json is not None:
+            self.id = json.get("id")
+            self.label = json.get("label")
+            self.output = safecast(json.get("output"), int)
+            self.icon = safecast(json.get("icon"), int)
+
     def __str__(self):
-        return "Component ( label='{}', address={}, output={}, type={} )" \
-            .format(self.label, self.address, self.output, self.type.value)
-
-    # To use it in a set, we need __eq__ and __hash__
-    def __eq__(self, other):
-        if isinstance(other, IngComponent):
-            return self.address == other.address and self.output == other.output and self.type.value == other.type.value
-        return False
-
-    def __hash__(self) -> int:
-        return hash((self.address, self.output, self.type.value))
+        return "Component ( id='{}', label='{}', output={} )" \
+            .format(self.id, self.label, self.output)
 
     def set_update_notify(self, callback: Callable):
         self._notify_update = [callback]
@@ -153,25 +164,26 @@ class IngComponent:
         for cb in self._notify_update:
             cb()
 
-    @property
-    def id(self):
-        return "{}_{}_{}".format(self.address, self.output, self.type.value)
-
 
 class IngObject:
     api: 'IngeniumAPI'
     is_knx: bool
     address: int
     type: IngComponentType
+    type_name: str
     components: List[IngComponent] = []
 
+    available: bool
+
     def __init__(self, api: 'IngeniumAPI', is_knx: bool, address: int, ctype: IngComponentType,
                  comps: List[IngComponent]):
         self.api = api
         self.is_knx = is_knx
         self.address = address
         self.type = ctype
+        self.type_name = ctype.name.replace("COD_", "").replace("COD", "")
         self.components = comps
+        self.available = False
 
     def __str__(self) -> str:
         return "IngObject ( address='{}', type={}, components={} )" \
@@ -180,6 +192,14 @@ class IngObject:
     async def update_state(self, p: Package) -> bool:
         raise NotImplementedError
 
+    def get_info(self):
+        return {
+            "name": ", ".join([x.label for x in self.components]),
+            "identifiers": {("ingenium", self.address)},
+            "model": self.type_name,
+            "manufacturer": "Ingenium",
+        }
+
 
 ACTUATOR_BLIND = 0
 ACTUATOR_NORMAL = 2
@@ -203,15 +223,10 @@ class IngActuator(IngObject):
         self.mode = ACTUATOR_BLIND if self.components[0].icon == 5 else ACTUATOR_NORMAL
 
     def get_switch_val(self, c: IngComponent):
-        if self.consumption != -1:
-            return None
         power = 1 << c.output
         return (self.output_state & power) == power
 
     async def action_switch(self, c: IngComponent):
-        if self.consumption != -1:
-            return None
-
         power = 1 << c.output
         d2 = c.output + 8 if (self.output_state & power) == power else c.output
         p = Package(0xFFFF, self.address, 4, 2, d2)
@@ -330,8 +345,12 @@ class IngActuator(IngObject):
                 self.output_state ^= (1 << p.data2)
                 return self.output_state != old_state
             elif p.data1 == 86:
+                # In case we get a weird zero when it's not a poewr meter
+                if self.consumption == -1 and p.data2 == 0:
+                    return False
+                factor = 230 / 10
                 old_cons = self.consumption
-                self.consumption = p.data2
+                self.consumption = p.data2 * factor
                 return self.consumption != old_cons
 
         return False
@@ -378,7 +397,7 @@ class IngMeterBus(IngObject):
         return [self.thresh1, self.thresh2, self.thresh3, self.thresh4][channel - 1]
 
     def get_available(self, channel: int):
-        return [self.available1, self.available2, self.available3, self.available4][channel - 1]
+        return self.available and [self.available1, self.available2, self.available3, self.available4][channel - 1]
 
     async def read_state(self):
         self.updating_channels = list(METERBUS_FIELDS.keys())
@@ -442,7 +461,7 @@ def convert_sif_value(value, mode: int):
     if mode == 0:
         return value / 5
     elif mode == 2:
-        return value > 0
+        return 1 if value > 0 else 0
     elif mode == 3:
         return value * 10
     elif mode == 4:
@@ -458,6 +477,8 @@ class IngSif(IngObject):
 
         # [Temperature, Noise(unimplemented), Presence, Illuminance, Humidity]
         self.values = [0, 0, False, 0, 0]
+        self.values_255 = [255, 255, 255, 255, 255]
+        self.values_len = 5
 
     async def read_state(self):
         self.updating_modes = SIF_AVAILABLE_MODES.copy()
@@ -475,22 +496,31 @@ class IngSif(IngObject):
             if len(self.updating_modes) > 0:
                 mode = self.updating_modes.pop()
 
-                old_value = self.values[mode]
-                self.values[mode] = convert_sif_value(p.data2, mode)
+                if mode < self.values_len:
+                    old_value = self.values[mode]
+                    self.values[mode] = convert_sif_value(p.data2, mode)
+                    self.values_255[mode] = p.data2
 
-                if len(self.updating_modes) > 0:
-                    await self._read()
+                    if len(self.updating_modes) > 0:
+                        await self._read()
 
-                return self.values[mode] != old_value
+                    return self.values[mode] != old_value
 
         if self.address != p.target:
             return False
 
+        if p.data1 > self.values_len - 1:
+            return False
+
         old_value = self.values[p.data1]
         self.values[p.data1] = convert_sif_value(p.data2, p.data1)
+        self.values_255[p.data1] = p.data2
 
         return self.values[p.data1] != old_value
 
+    def get_available(self, mode: int):
+        return self.available and self.values_255[mode] != 255
+
     def get_value(self, mode):
         return self.values[mode]
 
@@ -506,6 +536,9 @@ class IngBusingRegulator(IngObject):
         if self.address != p.target:
             return False
 
+        if p.data1 > 3:
+            return False
+
         old_value = self.channels[p.data1]
         self.channels[p.data1] = p.data2
 
@@ -538,6 +571,7 @@ class IngThermostat(IngObject):
             old = self.temp
             self.temp = p.data2 / 5
             return old != self.temp
+
         elif p.data1 == 1:
             old = self.set_point
             self.set_point = p.data2 / 5
@@ -546,12 +580,12 @@ class IngThermostat(IngObject):
         elif p.data1 == 10:
             old = self.cmode
             self.cmode = p.data2
-            return old != p.data2
+            return old != self.cmode
 
         elif p.data1 == 14:
             old = self.wmode
             self.wmode = p.data2
-            return old != p.data2
+            return old != self.wmode
 
         return False
 
